#!/bin/sh

modkext="AppleADBMouse.kext"
origkext="AppleADBMouse.kext"
origbundleid="com.apple.driver.AppleADBMouse"
backupdir="$HOME/Library/Application Support/iScroll2/Backup"
extdir="/System/Library/Extensions"

clear
cd "`dirname "$0"`"
echo
echo "Checking if trackpad driver is loaded..."
if ! kextstat | grep $origbundleid > /dev/null
then 
    echo "ERROR: Trackpad driver not loaded."
    echo "Your machine does not seem to have a compatible trackpad, "
    echo "or is using an alternative driver already."
    exit
fi
echo "Driver found."
echo
echo "Checking if trackpad supports W-enhanced mode..."
if ! ioreg -n AppleADBMouseType4 | grep "W Enhanced Trackpad" > /dev/null
then
    echo "ERROR: Your trackpad doesn't seem to support W-enhanced mode."
    echo "W-enhanced mode is necessary for two-finger scrolling to work."
    exit
fi
echo "W-enhanced mode supported."
echo
if ! [ -d ./$modkext ]
then
    echo "ERROR: Modified driver kernel extension not found."
    echo "Please run this script from the directory you unpacked the driver to."
    exit
fi
echo "This will PERMANENTLY activate the modified trackpad driver."
echo "A backup copy of the Apple-supplied original driver will be created in $backupdir."
echo "If you want to revert to using the original driver, use the supplied script or copy the backup to /System/Library/Extensions and REPAIR PERMISSIONS!"
echo
echo "Are you sure you want to continue?"
echo "Press CTRL-C or close this window to abort, RETURN to continue."
read
echo
echo "When prompted for a password, enter your admin password."
echo
echo "Backing up Apple-supplied original driver..."
if ! [ -d "$backupdir" ]
then
    if ! mkdir -p "$backupdir"
    then
        echo "ERROR: Unable to create $backupdir."
        exit
    fi
fi
if ! [ -d $extdir/$origkext ]
then
    echo "WARNING: Original driver not found."
    echo "No backup created."
else
    sudo chown -R `id -un`:`id -gn` $extdir/$origkext
    if ! sudo mv $extdir/$origkext "$backupdir"
    then
        echo "ERROR: Unable to move $origkext to $backupdir."
        exit
    fi
    echo "Backup of original driver created in $backupdir."
fi
echo
echo "Installing modified driver..."
if ! sudo cp -pR ./$modkext $extdir
then
    echo "ERROR: Unable to copy modified driver to $extdir."
    if ! sudo mv "$backupdir/$origkext" $extdir
    then
        echo "ERROR: Unable to restore original driver."
        echo "DO NOT TOUCH YOUR TRACKPAD NOW OR YOUR COMPUTER WILL CRASH!"
        exit
    fi
    sudo chown -R root:wheel $extdir/$origkext
    exit
fi
sudo chown -R root:wheel $extdir/$modkext
sudo chmod -R u=rwX $extdir/$modkext
sudo chmod -R go=rX $extdir/$modkext
echo "Modified driver installed to $extdir."
echo
echo "Do you want to activate the modified driver now?"
echo "Press CTRL-C or close this window to cancel (modified driver will be activated at next reboot), or press RETURN to activate the driver now."
read
echo
echo "Unloading the original driver..."
if ! sudo kextunload -b $origbundleid
then
    echo "ERROR: Unable to unload the original driver."
    exit
fi
echo "Original driver unloaded."
echo
echo "Loading the modified driver..."
if ! sudo kextload $extdir/$modkext
then
    echo "ERROR: Unable to load the modified driver."
    echo "Reloading the original driver."
    if ! sudo kextload "$backupdir/$origkext"
    then
        echo "ERROR: Unable to reload the original driver."
        echo "DO NOT TOUCH YOUR TRACKPAD NOW OR YOUR COMPUTER WILL CRASH!"
    fi
    exit
fi
echo "Driver replaced successfully."
echo