#!/bin/sh

modkext="AppleADBMouse.kext"
origkext="AppleADBMouse.kext"
origbundleid="com.apple.driver.AppleADBMouse"
modbundleid="com.apple.driver.AppleADBMouse"
backupdir="$HOME/Library/Application Support/iScroll2/Backup"
extdir="/System/Library/Extensions"

clear
echo
echo "Checking for backup of original driver..."
if ! [ -d "$backupdir/$origkext" ]
then
    echo "ERROR: Backup of original driver kernel extension not found."
    exit
fi
echo "Backup of original driver found."
echo
echo "This will restore the original Apple-supplied trackpad driver."
echo
echo "Are you sure you want to continue?"
echo "Press CTRL-C or close this window to abort, RETURN to continue."
read
echo
echo "When prompted for a password, enter your admin password."
echo
if [ -d $extdir/$modkext ]
then
    echo "Removing modified driver..."
    if ! sudo rm -R $extdir/$modkext
    then
        echo "ERROR: Unable to remove modified driver."
        exit
    fi
    echo "Modified driver removed."
    echo
fi
echo "Restoring backup of original driver to $extdir..."
if ! sudo mv "$backupdir/$origkext" $extdir
then
    echo "ERROR: Unable to restore original driver."
    exit
fi
sudo chown -R root:wheel $extdir/$origkext
sudo chmod -R u=rwX $extdir/$origkext
sudo chmod -R go=rX $extdir/$origkext
echo "Original driver restored."
echo
echo "Do you want to reactivate the original driver now?"
echo "Press CTRL-C or close this window to cancel (original driver will be activated at next reboot), or press RETURN to activate the driver now."
read
echo
echo "Unloading the modified driver..."
if ! sudo kextunload -b $modbundleid
then
    echo "ERROR: Unable to unload the modified driver."
    exit
fi
echo "Modified driver unloaded."
echo
echo "Loading the original driver..."
if ! sudo kextload -b $origbundleid
then
    echo "ERROR: Unable to load the original driver."
    exit
fi
echo "Driver replaced successfully."
echo